from __future__ import print_function

from .the_matrix import TheMatrix, DEFAULT_CURRENT_SOURCE_MA
from .detect import detect

import getopt, re, sys

# physical wiring
cs_pairs = [(cathode, anode) for cathode in range(12) for anode in [a for a in range(12) if a != cathode][:10]]

class LEDs(object):

    @classmethod
    def physical_layout(self):
        print("Physical layout:")
        print(("+" + "-"*11)*12 + "+")
        print("|"+("|".join([("/CS%d" % segment).center(11) for segment in range(12)])+"|"))
        print(("+" + "-"*5)*24 + "+")
        for y in range(5):
            line = ""
            for segment in range(12):
                segment_pairs = cs_pairs[segment*10:segment*10+10]
                line += "|" + ("CS%d" % segment_pairs[y][1]).center(5)
                line += "|" + ("CS%d" % segment_pairs[y+5][1]).center(5)
            print("%s|" % line)
            print(("+" + "-"*5)*24 + "+")
        print("")

    @classmethod
    def logical_layout(self):
        print("Logical layout:")
        print(("+" + "-"*11)*12 + "+")
        print("|"+("|".join([("Segment %X" % segment).center(11) for segment in range(12)])+"|"))
        print(("+" + "-"*5)*24 + "+")
        for y in range(5):
            line = ""
            for segment in range(12):
                first_segment_led = 16*segment + y
                line += "|" + ("%02X" % first_segment_led).center(5)
                line += "|" + ("%02X" % (first_segment_led + 5)).center(5)
            print("%s|" % line)
            print(("+" + "-"*5)*24 + "+")
        print("")

    @classmethod
    def display_leds(self, leds):
        """Display listed LEDs (logical number in hex or x,y coordinates in decimal)"""
        for m in matrix:
            m.reset()
            m.selectMemoryConfig(1)
            m.setCurrentSource(DEFAULT_CURRENT_SOURCE_MA)

            blinkPWMFrame = TheMatrix.BlinkPWMFrame()
            m.writeBlinkPWMFrame(0, blinkPWMFrame)

            onOffFrame = TheMatrix.OnOffFrame()
            for led in leds:
                x,y = 0,0
                coords = led.split(',')
                if len(coords) == 2:
                    x, y = [int(n) for n in coords]
                else:
                    match = re.match('^(/?)cs(\d+)$', led, re.IGNORECASE)
                    if match:
                        low = match.group(1)
                        signal = int(match.group(2))
                        connected_pairs = [i for i in range(len(cs_pairs)) if cs_pairs[i][0 if low else 1] == signal]
                        for pair_index in connected_pairs:
                            x = int(pair_index/5)
                            y = pair_index % 5
                            leds += ["%d,%d" % (x, y)]
                        continue
                    else:
                        led = int(led, 16)
                        hi = int(led/16)
                        lo = led % 16
                        x = hi*2 + int(lo/5)
                        y = lo % 5
                assert(x in range(24))
                assert(y in range(5))
                onOffFrame.setPixel(x, y)
            m.writeOnOffFrame(0, onOffFrame)

            m.setDisplayOptions()
            m.display(1)

            m.displayPictureFrame(0)

def usage():
    print("Usage: {} [-a <address>[,<address>...]] [-h] [-l] [-p] <led_numbers>".format(sys.argv[0]), file=sys.stderr)

def main(args):
    global matrix

    addresses = []

    try:
        opts, args = getopt.getopt(args, "hlpa:")
    except getopt.GetoptError:
        usage()
        sys.exit(2)

    if len(args+opts) == 0:
        usage()

    for opt, arg in opts:
        if opt == '-h':
            usage()
            sys.exit()
        elif opt == '-l':
            LEDs.logical_layout()
        elif opt == '-p':
            LEDs.physical_layout()
        elif opt == '-a':
            addresses = [int(address, 16) for address in arg.split(',')]

    if len(opts)>0 and len(args)==0:
        sys.exit(0)

    if len(addresses) == 0:
        addresses = detect()

    matrix = [TheMatrix(address) for address in addresses]

    LEDs.display_leds(args)

def command_line():
    main(sys.argv[1:])

if __name__ == "__main__":
    command_line(sys.argv[1:])



import unittest

from the_matrix.layout import Layout
from collections import namedtuple

class TestLayout(unittest.TestCase):
    """Test Layout class"""

    def test_create_default(self):
        """Check default layout size"""
        layout = Layout()
        self.assertEqual(layout.width, 24)
        self.assertEqual(layout.height, 5)

    def test_create_24x5(self):
        """Check 24x5 layout size"""
        layout = Layout(width=24, height=5)
        self.assertEqual(layout.width, 24)
        self.assertEqual(layout.height, 5)

    def test_create_12x11(self):
        """Check 12x11 layout size"""
        layout = Layout(width=12, height=11)
        self.assertEqual(layout.width, 12)
        self.assertEqual(layout.height, 11)

    def test_create_bad_layout(self):
        """Check refusal to create layout of bad size"""
        with self.assertRaises(AssertionError):
            layout = Layout(width=10, height=5)

    def test_not_reversed_by_default(self):
        """Check LEDs in layout are not reversed by default"""
        layout = Layout()
        self.assertFalse(layout.reversed)

    def test_layout_reversed_on_request(self):
        """Check LEDs in layout are reversed on request"""
        layout = Layout(reversed=True)
        self.assertTrue(layout.reversed)

    def test_coords_out_of_range(self):
        """Check bad coordinates are rejected"""
        layout = Layout()
        with self.assertRaises(AssertionError):
            led = layout.ledAt(-1, 0)
        with self.assertRaises(AssertionError):
            led = layout.ledAt(0, -1)
        with self.assertRaises(AssertionError):
            led = layout.ledAt(layout.width, 0)
        with self.assertRaises(AssertionError):
            led = layout.ledAt(0, layout.height)

    def _check_expected_leds(self, layout, expected_leds):
        for expected_led, x, y in expected_leds:
            led = layout.ledAt(x, y)
            self.assertEqual(led, expected_led, 'should get LED 0x%02X at %d,%d' % (expected_led, x, y))

    def test_led_for_good_coords_24x5(self):
        """Check good coordinates return an LED for 24x5 matrix"""
        layout = Layout(24, 5)
        expected_leds = [
            [0x00, 0, 0],
            [0xb5, 23, 0],
            [0x09, 1, 4],
            [0xb4, 22, 4],
            [0x57, 11, 2],
        ]
        self._check_expected_leds(layout, expected_leds)

    def test_led_for_good_coords_12x11(self):
        """Check good coordinates return an LED for 12x11 matrix"""
        layout = Layout(12, 11)
        expected_leds = [
            [0x00, 0, 0],
            [0xb0, 11, 0],
            [0x0a, 0, 10],
            [0xba, 11, 10],
            [0x53, 5, 3],
        ]
        self._check_expected_leds(layout, expected_leds)

    def test_led_for_good_coords_24x5_reversed(self):
        """Check good coordinates return an LED for 24x5 reverse-connected matrix"""
        layout = Layout(24, 5, reversed=True)
        expected_leds = [
            [0x10, 0, 0],
            [0x5a, 23, 0],
            [0xa0, 1, 4],
            [0x4a, 22, 4],
            [0x85, 11, 2],
        ]
        self._check_expected_leds(layout, expected_leds)

    def test_addresses_for_led_number(self):
        """Check address mapping for LED by number"""
        layout = Layout()
        LED = namedtuple('LED', 'number on_off_byte on_off_bit blink_byte blink_bit pwm_byte')
        expected_leds = [
            LED(0x00, 0x00, 0, 0x00, 0, 0x18),
            LED(0x21, 0x04, 1, 0x04, 1, 0x2f),
            LED(0x29, 0x05, 1, 0x05, 1, 0x37),
            LED(0x3a, 0x07, 2, 0x07, 2, 0x43),
        ]
        for e in expected_leds:
            self.assertEqual(layout.numToAddressOnOff(e.number), (e.on_off_byte, e.on_off_bit), 'should get correct on/off address for led %02X' % e.number)
            self.assertEqual(layout.numToAddressBlink(e.number), (e.blink_byte, e.blink_bit), 'should get correct blink address for led %02X' % e.number)
            self.assertEqual(layout.numToAddressPWM(e.number), e.pwm_byte, 'should get correct PWM address for led %02X' % e.number)

if __name__ == '__main__':
    unittest.main()



from __future__ import print_function

from .the_matrix import TheMatrix, DEFAULT_CURRENT_SOURCE_MA
from .pixel_font import *
from .detect import detect

import getopt, math, re, sys

class ScrollText(object):
    """Class to provide scrolling text for The Matrix"""

    @classmethod
    def display_banner(self, args):
        frames = [TheMatrix.OnOffFrame() for _ in range(len(matrix))]
        font = PixelFont5Narrow()
        pixel_data = font.string(' '.join(args))

        # check message length, allowing for one extra blank frame
        num_frames = int(math.ceil(len(pixel_data[0]) / 24.0))
        assert num_frames < 36, 'message too long to fit in 36 frames'

        if len(pixel_data[0]) % 24 != 0:
            for y in range(5):
                pixel_data[y] += '.' * (24 - (len(pixel_data[y]) % 24))

        for i in range(num_frames):
            frame = TheMatrix.OnOffFrame()
            for y in range(5):
                pixel_row = pixel_data[y][24*i:24*i+24]
                for x in range(24):
                    if pixel_row[x] == '#':
                        frame.setPixel(x, y)
            frames.append(frame)

        # turn displays off first to avoid losing sync
        for i in range(len(matrix)):
            matrix[i].display(0)

        for i in range(len(matrix)):
            m = matrix[i]
            m.reset()
            m.selectMemoryConfig(1)
            m.setClockSync(sync_out=1 if i==0 else 0, sync_in=0 if i==0 else 1)
            m.setCurrentSource(DEFAULT_CURRENT_SOURCE_MA)
            blinkPWMFrame = TheMatrix.BlinkPWMFrame()
            m.writeBlinkPWMFrame(0, blinkPWMFrame)
            for frame_num in range(len(frames)):
                m.writeOnOffFrame(frame_num, frames[frame_num])
            for frame_num in range(min(len(matrix), 36-len(frames))):
                m.writeOnOffFrame(frame_num + len(frames), TheMatrix.OnOffFrame())
            m.setDisplayOptions(loops=7)
            m.setMovieMode(frames=len(frames))
            m.setFrameTime(1, enable_scrolling=1)

        for i in range(len(matrix)):
            m = matrix[i]
            m.display(1)
            m.displayMovie(frame=i)

def usage():
    print("Usage: {} [-a <address>[,<address>...]] <text>".format(sys.argv[0]), file=sys.stderr)

def main(args):
    global matrix

    addresses = []

    try:
        opts, args = getopt.getopt(args, 'ha:')
    except getopt.GetoptError:
        usage()
        sys.exit(2)

    for opt, arg in opts:
        if opt == '-h':
            usage()
            sys.exit()
        if opt == '-a':
            addresses = [int(address, 16) for address in arg.split(',')]

    if len(args) < 1:
        usage()
        sys.exit(2)

    if len(addresses) == 0:
        addresses = detect()

    matrix = [TheMatrix(address) for address in addresses]

    ScrollText.display_banner(args)

def command_line():
    main(sys.argv[1:])



class SMBus(object):
    """Fake SMBus for debugging purposes"""
    is_fake = True

    def __init__(self, bus_number=1):
        print("BUS:INIT bus=%d" % bus_number)

    def write_byte_data(self, address, register, data):
        print("BUS:WRITE [0x%02X 0x%02X 0x%02X]" % (address, register, data))

    def write_i2c_block_data(self, address, register, data):
        print("BUS:WRITE_BLOCK [0x%02X 0x%02X %s]" % (address, register, ' '.join(list(map(lambda x: "%02X" % x, data)))))

    def read_byte(self, address):
        print("BUS:READ  [0x%02x]" % address)



from flask import Flask, render_template, request
from .the_matrix import TheMatrix, DEFAULT_CURRENT_SOURCE_MA
from .layout import Layout
from .detect import detect

import re

app = Flask(__name__)

addresses = detect()
matrices = dict((address, TheMatrix(address)) for address in addresses)

blinkPWMFrames = dict((address, None) for address in addresses)
onOffFrames = dict((address, None) for address in addresses)
isReversed = dict((address, False) for address in addresses)
ledCurrents = dict((address, DEFAULT_CURRENT_SOURCE_MA) for address in addresses)
resetCurrents = dict((address, DEFAULT_CURRENT_SOURCE_MA) for address in addresses)

cs_pairs = [(anode, cathode) for cathode in range(12) for anode in [a for a in range(12) if a != cathode][:10]]
chip = [{'label': label} for label in 'GND CS6 VDD CS7 CS8 CS9 GND CS11 CS10 VDD IRQ SYNC RSTN GND SCL SDA ADDR VDD VDD CS5 CS4 GND CS2 CS3 CS0 VDD CS1 GND'.split()]
for pin in chip:
    match = re.match('^CS(\d+)', pin['label'])
    if match:
        signal = int(match.group(1))
        pin['clickable'] = True
        # get anode-connected LEDs
        anode_led_indices = [i for i in range(len(cs_pairs)) if cs_pairs[i][0] == signal]
        pin['anode_leds'] = [[int(i/5), i%5] for i in anode_led_indices]
        # get cathode-connected LEDs
        cathode_led_indices = [i for i in range(len(cs_pairs)) if cs_pairs[i][1] == signal]
        pin['cathode_leds'] = [[int(i/5), i%5] for i in cathode_led_indices]

def updateFrame(address, frameNumber):
    matrices[address].writeOnOffFrame(frameNumber, onOffFrames[address])
    return ""

@app.route('/')
def main_route():
    pixels = dict((address, [[onOffFrames[address].getPixel(x, y) for x in range(24)] for y in range(5)]) for address in matrices.keys())
    return render_template(
        'the_matrix.html',
        width=24,
        height=5,
        current=ledCurrents,
        resetCurrents=resetCurrents,
        pixels=pixels,
        chip=chip,
        reversed=isReversed,
        addresses=sorted(matrices.keys())
    )

@app.route('/reset', methods=['POST'])
def reset():
    try:
        addresses = [int(request.form['address'])]
    except:
        addresses = matrices.keys()

    for address in addresses:
        isReversed[address] = False
        matrix = matrices[address]

        matrix.reset()
        matrix.selectMemoryConfig(1)

        ledCurrents[address] = DEFAULT_CURRENT_SOURCE_MA
        matrix.setCurrentSource(ledCurrents[address])

        layout = Layout(reversed=isReversed[address])
        blinkPWMFrames[address] = TheMatrix.BlinkPWMFrame(layout=layout)
        matrix.writeBlinkPWMFrame(0, blinkPWMFrames[address])

        onOffFrames[address] = TheMatrix.OnOffFrame(layout=layout)
        matrix.writeOnOffFrame(0, onOffFrames[address])

        matrix.setDisplayOptions()
        matrix.display(1)
        matrix.displayPictureFrame(0)

    return ""

@app.route('/setCurrent', methods=['POST'])
def setCurrent():
    address = int(request.form['address'])
    current = int(request.form['current'])

    ledCurrents[address] = current

    matrices[address].setCurrentSource(ledCurrents[address])

    return ""

@app.route('/allOff', methods=['POST'])
def allOff():
    address = int(request.form['address'])

    blinkPWMFrames[address] = TheMatrix.BlinkPWMFrame()
    onOffFrames[address] = TheMatrix.OnOffFrame(0)

    matrix = matrices[address]
    matrix.writeBlinkPWMFrame(0, blinkPWMFrames[address])
    matrix.writeOnOffFrame(0, onOffFrames[address])

    return ""

@app.route('/allOn', methods=['POST'])
def allOn():
    address = int(request.form['address'])

    blinkPWMFrames[address] = TheMatrix.BlinkPWMFrame()
    onOffFrames[address] = TheMatrix.OnOffFrame(1)

    matrix = matrices[address]
    matrix.writeBlinkPWMFrame(0, blinkPWMFrames[address])
    matrix.writeOnOffFrame(0, onOffFrames[address])

    return ""

@app.route('/setPixel', methods=['POST'])
def setPixel():
    address = int(request.form['address'])

    for coords in request.form.getlist('coords[]'):
        x, y = [int(i) for i in coords.split(',')]
        onOffFrames[address].setPixel(x, y)

    return updateFrame(address, 0)

@app.route('/clearPixel', methods=['POST'])
def clearPixel():
    address = int(request.form['address'])

    for coords in request.form.getlist('coords[]'):
        x, y = [int(i) for i in coords.split(',')]
        onOffFrames[address].setPixel(x, y, 0)

    return updateFrame(address, 0)

@app.route('/setReversed', methods=['POST'])
def setReversed():
    address = int(request.form['address'])
    reversedFlag = request.form['reversed']

    newReversed = reversedFlag == 'true'
    if newReversed != isReversed[address]:
        isReversed[address] = newReversed
        newLayout = Layout(reversed=newReversed)
        blinkPWMFrames[address].layout = newLayout
        onOffFrames[address].layout = newLayout

        matrix = matrices[address]
        matrix.writeBlinkPWMFrame(0, blinkPWMFrames[address])
        matrix.writeOnOffFrame(0, onOffFrames[address])

    return ""

def main():
    reset()
    app.run(host='0.0.0.0')

if __name__ == "__main__":
    main()



class Layout(object):
    """Class to represent the layout of an AS1130 matrix"""

    def __init__(self, width=24, height=5, reversed=False):
        assert(width in [24, 12])
        assert(height == (5 if width==24 else 11))
        assert(reversed in ([True,False] if width == 24 else [False]))
        self._width = width
        self._height = height
        self._reversed = reversed
        self._connections = None
        self._mappedLayout = None

    @property
    def width(self):
        """Number of columns in LED matrix"""
        return self._width

    @property
    def height(self):
        """Number of rows in LED matrix"""
        return self._height

    @property
    def reversed(self):
        """Flag to indicate all LEDs connected in reverse"""
        return self._reversed

    @property
    def connections(self):
        """Connection pairs for each LED in coordinate grid"""
        if self._connections is None:
            # get connection pairs
            w = 10 if self.width == 24 else 11
            conn = [(anode, cathode) for cathode in range(12) for anode in [a for a in range(12) if a!= cathode][:w]]
            # arrange connection pairs in coordinate grid
            col_height, cols = (5, 24) if self.width == 24 else (11, 12)
            self._connections = [conn[col_height*i:col_height*i+col_height] for i in range(cols)]
        return self._connections

    def ledAt(self, x, y):
        """Returns the number of the LED at the specified coordinates"""
        assert(x in range(self.width))
        assert(y in range(self.height))
        anode, cathode = self.connections[x][y]
        if self.reversed:
            if self._mappedLayout == None:
                self._mappedLayout = Layout(12, 11)
            mappedX = anode
            mappedY = self._mappedLayout.connections[anode].index((cathode, anode))
            number = self._mappedLayout.ledAt(mappedX, mappedY)
        else:
            number = 16*cathode + y + (5*(x % 2) if self.width == 24 else 0)
        return number

    def numToAddressOnOff(self, led_number):
        """Returns the on/off address details (byte and bit) of the LED with the specified number"""
        hi, lo = ((led_number >> 4) & 0xf), (led_number & 0xf)
        assert(hi in range(12))
        assert(lo in range(11))
        byte_offset = int(lo/8) # later bits in second byte
        address_byte = 2*hi + int(lo/8)
        address_bit = lo % 8
        return (address_byte, address_bit)

    def numToAddressBlink(self, led_number):
        """Returns the blink address details (byte and bit) of the LED with the specified number"""
        hi, lo = ((led_number >> 4) & 0xf), (led_number & 0xf)
        assert(hi in range(12))
        assert(lo in range(11))
        byte_offset = int(lo/8) # later bits in second byte
        address_byte = 2*hi + int(lo/8)
        address_bit = lo % 8
        return (address_byte, address_bit)

    def numToAddressPWM(self, led_number):
        """Returns the PWM address details (byte) of the LED with the specified number"""
        hi, lo = ((led_number >> 4) & 0xf), (led_number & 0xf)
        assert(hi in range(12))
        assert(lo in range(11))
        address_byte = 0x18 + 11*hi + lo
        return address_byte



from __future__ import print_function

from .the_matrix import TheMatrix, DEFAULT_CURRENT_SOURCE_MA
from .pixel_font import *
from .detect import detect

import getopt, sys

def usage():
    print("Usage: {} [-a <address>[,<address>...]]".format(sys.argv[0]), file=sys.stderr)

def main(args):
    addresses = []

    try:
        opts, args = getopt.getopt(args, 'ha:')
    except getopt.GetoptError:
        usage()
        sys.exit(2)

    for opt, arg in opts:
        if opt == '-h':
            usage()
            sys.exit()
        if opt == '-a':
            addresses = [int(address, 16) for address in arg.split(',')]

    if len(addresses) == 0:
        addresses = detect()

    font = PixelFont5Narrow()

    for address in addresses:
        matrix = TheMatrix(address)
        pixel_data = font.string('0x%02x' % address)
        if len(pixel_data[0]) % 24 != 0:
            for y in range(5):
                pixel_data[y] += '.' * (24 - (len(pixel_data[y]) % 24))

        frame = TheMatrix.OnOffFrame()
        for y in range(5):
            pixel_row = pixel_data[y][0:24]
            for x in range(24):
                if pixel_row[x] == '#':
                    frame.setPixel(x, y)

        matrix.display(0)
        matrix.reset()
        matrix.selectMemoryConfig(1)
        matrix.setCurrentSource(DEFAULT_CURRENT_SOURCE_MA)
        blinkPWMFrame = TheMatrix.BlinkPWMFrame()
        matrix.writeBlinkPWMFrame(0, blinkPWMFrame)
        matrix.writeOnOffFrame(0, frame)
        matrix.setDisplayOptions(loops=0)
        matrix.display(1)
        matrix.displayPictureFrame()

def command_line():
    main(sys.argv[1:])



import math, time
from .layout import Layout
from .detect import detect

try:
    from smbus import SMBus
except:
    from .fake_smbus import SMBus

DEFAULT_CURRENT_SOURCE_MA = 5

class TheMatrix():
    """Class to control Boldport 'The Matrix'"""
    class _register:
        nop             = 0x00
        on_off_frame    = 0x01
        blink_pwm_frame = 0x40
        dot_correction  = 0x80
        control         = 0xc0
        select          = 0xfd

    class _control:
        picture         = 0x00
        movie           = 0x01
        movie_mode      = 0x02
        frame_time      = 0x03
        display_option  = 0x04
        current_source  = 0x05
        config          = 0x06
        interrupt_mask  = 0x07
        interrupt_frame = 0x08
        shutdown        = 0x09
        i2c_monitor     = 0x0a
        clock_sync      = 0x0b
        interrupt_stat  = 0x0e
        status          = 0x0f
        open_led        = 0x20

    def __init__(self, address=None, bus_number=1):
        """Create TheMatrix object with specified I2C address and bus number"""
        if address == None:
            address = detect()[0]

        self._address = address
        self._bus = SMBus(bus_number)

    def _writeCommand(self, register, subregister, data):
        self._bus.write_byte_data(self._address, self._register.select, register)
        if not isinstance(data, list):
            data = [data]
        for i in range(0, len(data), 32):
            self._bus.write_i2c_block_data(self._address, subregister+i, data[i:i+32])

    def readByte(self, register, subregister):
        """Read byte from specified register and subregister"""
        self._bus.write_byte_data(self._address, self._register.select, register)
        self._bus.write_byte(self._address, subregister)
        return self._bus.read_byte(self._address)

    def reset(self):
        """Reset TheMatrix display driver chip"""
        self._writeCommand(self._register.control, self._control.shutdown, 0)
        time.sleep(0.005) # wait 5ms to chip to be ready

    def _configure(self,
            low_vdd_rst=0,
            low_vdd_stat=0,
            led_error_correction=0,
            dot_corr=0,
            common_addr=0,
            mem_conf=0):
        data = (
            (low_vdd_rst & 1) << 7 |
            (low_vdd_stat & 1) << 6 |
            (led_error_correction & 1) << 5 |
            (dot_corr & 1) << 4 |
            (common_addr & 1) << 3 |
            (mem_conf & 7)
        )
        self._writeCommand(self._register.control, self._control.config, data)

    def selectMemoryConfig(self, number=1):
        """Select RAM configuration"""
        self._configure(mem_conf=number)

    def setCurrentSource(self, mA=DEFAULT_CURRENT_SOURCE_MA):
        """Set current source (milliamps)"""
        assert(mA >= 0 and mA <= 30)
        data = int(mA*(255/30.0))
        self._writeCommand(self._register.control, self._control.current_source, data)

    def setDisplayOptions(self, loops=1, blink_freq=0, scan_limit=11):
        """Set display options"""
        data = (
            (loops & 7) << 5 |
            (blink_freq & 1) << 4 |
            (scan_limit & 15)
        )
        self._writeCommand(self._register.control, self._control.display_option, data)

    def setClockSync(self, clk_out=0, sync_out=0, sync_in=0):
        """Set sync options"""
        data = (
            (clk_out & 3) << 2 |
            (sync_out & 1) << 1 |
            (sync_in & 1)
        )
        self._writeCommand(self._register.control, self._control.clock_sync, data)

    def displayPictureFrame(self, number=0, display=1, blink=0):
        """Display picture frame"""
        data = (
            (blink & 1) << 7 |
            (display & 1) << 6 |
            (number & 63)
        )
        self._writeCommand(self._register.control, self._control.picture, data)

    def displayMovie(self, frame=0, start=1, blink=0):
        """Display movie"""
        assert(frame <= 35)
        data = (
            (blink & 1) << 7 |
            (start & 1) << 6 |
            (frame & 63)
        )
        self._writeCommand(self._register.control, self._control.movie, data)

    def setMovieMode(self, frames=1, blink=0, end_last=0):
        """Set movie play options"""
        assert(frames >= 1 and frames <= 36)
        data = (
            (blink & 1) << 7 |
            (end_last & 1) << 6 |
            (frames - 1) & 63
        )
        self._writeCommand(self._register.control, self._control.movie_mode, data)

    def setFrameTime(self, delay=1, scroll_dir=1, block_size=1, enable_scrolling=0):
        """Set movie frame time in units of 32.5ms"""
        assert(delay >= 0 and delay <= 15)
        frame_fad = 0
        data = (
            (frame_fad & 1) << 7 |
            (scroll_dir & 1) << 6 |
            (block_size & 1) << 5 |
            (enable_scrolling & 1) << 4 |
            (delay & 15)
        )
        self._writeCommand(self._register.control, self._control.frame_time, data)

    def display(self, on=1):
        """Set display on/off"""
        test_all = 0
        auto_test = 0
        manual_test = 0
        init = 1
        data = (
            (test_all & 1) << 4 |
            (auto_test & 1) << 3 |
            (manual_test & 1) << 2 |
            (init & 1) << 1 |
            (on & 1)
        )
        self._writeCommand(self._register.control, self._control.shutdown, data)

    def writeBlinkPWMFrame(self, number, frameData):
        """Write a blink/PWM frame to the specified frame number"""
        data = frameData.data
        self._writeCommand(self._register.on_off_frame+number, 0, data)

    def writeOnOffFrame(self, number, frameData):
        """Write an on/off frame to the specified frame number"""
        data = frameData.data
        self._writeCommand(self._register.on_off_frame+number, 0, data)

    class OnOffFrame():
        def __init__(self, value=0, pwm=0, layout=Layout()):
            value &= 1
            byte0 = int(''.join([str(value) for i in range(8)]), 2)
            byte1 = byte0 & 7
            self._pixels = [byte0, byte1] * 12
            self.pwm = pwm
            self._layout = layout

        @property
        def pwm(self):
            return (self._pixels[1] >> 5) & 7

        @pwm.setter
        def pwm(self, value):
            self._pixels[1] = (self._pixels[1] & 0x1f) | (value << 5)

        def setPixel(self, x, y, value=1):
            value &= 1
            led_number = self.layout.ledAt(x, y)
            address_byte, address_bit = self.layout.numToAddressOnOff(led_number)
            masked = 1 << address_bit
            if value == 1:
                self._pixels[address_byte] |= masked
            else:
                self._pixels[address_byte] &= ~masked & 255

        def getPixel(self, x, y):
            led_number = self.layout.ledAt(x, y)
            address_byte, address_bit = self.layout.numToAddressOnOff(led_number)
            value = (self._pixels[address_byte] >> address_bit) & 1
            return value

        @property
        def data(self):
            return self._pixels

        @property
        def layout(self):
            return self._layout

        @layout.setter
        def layout(self, newLayout):
            self._layout = newLayout

        def __repr__(self):
            output = ''
            for y in range(5):
                for x in range(24):
                    value = self.getPixel(x, y)
                    output += '#' if value == 1 else '.'
                output += "\n"
            return output

    class BlinkPWMFrame():
        def __init__(self, blink=0, pwm=255, layout=Layout()):
            blink &= 1
            pwm &= 255
            blink_byte0 = int(''.join([str(blink) for i in range(8)]), 2)
            blink_byte1 = blink_byte0 & 7
            blink_bytes = [blink_byte0, blink_byte1] * 12
            pwm_bytes = [pwm] * 12*11
            self._data = blink_bytes + pwm_bytes
            self._layout = layout

        @property
        def data(self):
            return self._data

        @property
        def layout(self):
            return self._layout

        @layout.setter
        def layout(self, newLayout):
            self._layout = newLayout

        def setBlink(self, x, y, value=1):
            value &= 1
            led_number = self.layout.ledAt(x, y)
            address_byte, address_bit = self.layout.numToAddressBlink(led_number)
            masked = 1 << address_bit
            if value == 1:
                self._data[address_byte] |= masked
            else:
                self._pixels[address_byte] &= ~masked & 255

        def setPWM(self, x, y, value=255):
            value &= 255
            led_number = self.layout.ledAt(x, y)
            address_byte = self.layout.numToAddressPWM(led_number)
            self._data[address_byte] = value



from setuptools import setup, find_packages
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='the_matrix',
    packages=find_packages(exclude=['tests', 'docs']),
    entry_points = {
        'console_scripts': [
            'the_matrix_identify=the_matrix.identify:command_line',
            'the_matrix_leds=the_matrix.leds:command_line',
            'the_matrix_scrolltext=the_matrix.scrolltext:command_line',
            'the_matrix_web=the_matrix.web:main',
        ]
    },
    include_package_data=True,
    zip_safe=False,

    use_scm_version=True,
    setup_requires=['setuptools_scm'],
    install_requires=['flask'],

    description='Module to control Boldport "The Matrix"',
    long_description=long_description,
    url='https://github.cm/threebytesfull.com/the_matrix_pi',

    author='Rufus Cable',
    author_email='rufus@threebytesfull.com',

    license='MIT',

    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Embedded Systems',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],

    keywords='boldport led matrix i2c rpi as1130',
)



try:
    from smbus import SMBus
except:
    from .fake_smbus import SMBus

def detect(bus_number=1):
    bus = SMBus(bus_number)
    if hasattr(bus, 'is_fake'):
        return [0x30]

    addresses = [];
    for address in range(0x30, 0x38):
        try:
            result = bus.read_byte(address)
            addresses.append(address)
        except:
            next
    return addresses
